local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

local resourceCostDefault = 100

local costsMemory = { }

function Capitalist:CostMemoryReset()
	costsMemory = { }
end

function Capitalist:SetResourceGoldCost(resourceName, cost)
	self.db.profile.components[resourceName].cost = cost
end

function Capitalist:GetResourceGoldCost(resourceName)
	if (self.db.profile.components and
		self.db.profile.components[resourceName] and
		self.db.profile.components[resourceName].cost) then
		return self.db.profile.components[resourceName].cost
	else
		return resourceCostDefault
	end
end

function Capitalist:GetResourceCopperCost(resourceName)
	return self:GetResourceGoldCost(resourceName) * 10000
end

function Capitalist:GetItemCopperCost(itemName)
	if (self:IsResource(itemName)) then
		return self:GetResourceCopperCost(itemName)
	else
		local skillName, profession = self:SkillNameFromProductName(itemName)
		return self:CalculateSkillCost(profession, skillName)
	end
end

function Capitalist:GetItemGoldCost(itemName)
	return self:GetItemCopperCost(itemName) / 10000
end

--REVIEW: Remove or formalize statistics information
Capitalist.hit = 0
Capitalist.miss = 0
function Capitalist:CalculateSkillCost(profession, skillName)
	local cost = costsMemory[skillName]

	if (not cost) then
		Capitalist.miss = Capitalist.miss + 1
		cost = 0

		for componentName, componentCount in pairs(self.db.profile.professions[profession][skillName].components) do
			local componentCostInCopper

			if (self:IsResource(componentName)) then
				componentCostInCopper = self:GetResourceGoldCost(componentName) * 10000
			else
				-- TODO: use all skills, find the cheapest
				local componentSkillName, componentProfession = self:SkillNameFromProductName(componentName)
				componentCostInCopper = self:CalculateSkillCost(componentProfession, componentSkillName)
			end

			cost = cost + componentCount * componentCostInCopper
		end

		costsMemory[skillName] = cost
	else
		Capitalist.hit = Capitalist.hit + 1
	end

	return cost
end

function Capitalist:OutputItemCost(itemName)
	local skills = self:SkillsFromProductName(itemName)
	if (skills) then
		for skillName, profession in pairs(skills) do
			local itemText = self:ProductTextFromSkillName(profession, skillName)
			local itemCost = self:CalculateSkillCost(profession, skillName)	
			local recipeText = self:SkillTextFromSkillName(profession, skillName)
			self:Print(string.format(L["Product %s costs %s to make via %s."], itemText, self:PrettyCurrency(itemCost), recipeText))
		end
	elseif (self.db.profile.components[itemName]) then
		local itemText = self:GetComponentText(itemName)
		local itemCost = self:GetResourceGoldCost(itemName) * 10000
		local itemMerchant = self:ResourceGetMerchant(itemName)
		if (itemMerchant) then
			self:Print(string.format(L["Resource %s cost is set at %s, sold by %s."], itemText, self:PrettyCurrency(itemCost), itemMerchant))
		else
			self:Print(string.format(L["Resource %s cost is set at %s."], itemText, self:PrettyCurrency(itemCost)))
		end
	else
		self:Print(string.format(L["Item %s value unknown."], itemName))
	end
end
